from .temperature_logger_abstract import TemperatureLoggerAbstract
from .tc08usb import TC08USB, USBTC08_TC_TYPE


class TemperatureLogger(TemperatureLoggerAbstract):
    """
    USB temperature logger TC08 (with 8 thermocouples) 
    """

    def __init__(self):
        """
        constructor
        """
        self._usb_conn = TC08USB()

    def connect(self):
        """
        Connect to pico thermo logger device

        Returns:
            bool: connection status - True if connected, else False
        """
        is_connected = self._usb_conn.open_unit()
        return bool(is_connected)

    def set_channel(self, channel_number):
        """
        Set thermo couple channel

        Args:
            channel_number (int): thermo couple channel
        """
        self._usb_conn.set_mains(50)
        self._usb_conn.set_channel(channel_number, USBTC08_TC_TYPE.K)

    def read_temperature(self, channel):
        """
        Read thermocouple logger output

        Args:
            channel (int): thermo couple channel

        Returns:
            float: temperature value read
        """
        self._usb_conn.get_single()
        return self._usb_conn[channel]

    def close_connection(self):
        """
        Close connection to device
        """
        self._usb_conn.close_unit()



from abc import ABC, abstractmethod


class TemperatureLoggerAbstract(ABC):
    @abstractmethod
    def connect(self):
        pass
    
    @abstractmethod
    def read_temperature(self, channel: int):
        pass
    
    @abstractmethod
    def close_connection(self):
        pass
    


import unittest
from unittest.mock import patch, MagicMock
from opsys_temperature_logger.temperature_logger import TemperatureLogger


class Test(unittest.TestCase):
    @ classmethod
    def setUp(self):
        pass

    @ classmethod
    def setUpClass(cls):
        pass

    @ classmethod
    def tearDownClass(cls):
        pass

    @ patch.object(TemperatureLogger, 'connect')
    def test_connect(self, pico_mock: MagicMock):
        pico = TemperatureLogger()
        pico.connect()
        pico_mock.assert_called_once_with()

    @ patch.object(TemperatureLogger, 'set_channel')
    def test_set_channel(self, pico_mock: MagicMock):
        pico = TemperatureLogger()
        channel = 3
        pico.set_channel(channel=channel)
        pico_mock.assert_called_once_with(channel=3)

    @ patch.object(TemperatureLogger, 'read_temperature')
    def test_read_temperature(self, pico_mock: MagicMock):
        pico = TemperatureLogger()
        channel = 3
        pico.read_temperature(channel=channel)
        pico_mock.assert_called_once_with(channel=3)

    @ patch.object(TemperatureLogger, 'close_connection')
    def test_close_connection(self, pico_mock: MagicMock):
        pico = TemperatureLogger()
        pico.close_connection()
        pico_mock.assert_called_once_with()



from enum import Enum
import six
import ctypes
import numpy as np
import os


class USBTC08_ERROR(Enum):
    OK = 0
    OS_NOT_SUPPORTED = 1
    NO_CHANNELS_SET = 2
    INVALID_PARAMETER = 3
    VARIANT_NOT_SUPPORTED = 4
    INCORRECT_MODE = 5
    ENUMERATION_INCOMPLETE = 6
    NOT_RESPONDING = 7
    FW_FAIL = 8
    CONFIG_FAIL = 9
    NOT_FOUND = 10
    THREAD_FAIL = 11
    PIPE_INFO_FAIL = 12
    NOT_CALIBRATED = 13
    PICOPP_TOO_OLD = 14
    COMMUNICATION = 15

    @classmethod
    def help(cls, error):
        d = {
            USBTC08_ERROR.OK: "No error occurred.",
            USBTC08_ERROR.OS_NOT_SUPPORTED: "The driver supports Windows XP SP3, Windows Vista, Windows 7 and Windows 8.",
            USBTC08_ERROR.NO_CHANNELS_SET: "A call to usb_tc08_set_channel() is required.",
            USBTC08_ERROR.INVALID_PARAMETER: "One or more of the function arguments were invalid.",
            USBTC08_ERROR.VARIANT_NOT_SUPPORTED: "The hardware version is not supported. Download the latest driver.",
            USBTC08_ERROR.INCORRECT_MODE: "An incompatible mix of legacy and non-legacy functions was called (or usb_tc08_get_single() was called while in streaming mode.)",
            USBTC08_ERROR.ENUMERATION_INCOMPLETE: "usb_tc08_open_unit_async() was called again while a background enumeration was already in progress.",
            USBTC08_ERROR.NOT_RESPONDING: "Cannot get a reply from a USB TC-08.",
            USBTC08_ERROR.FW_FAIL: "Unable to download firmware.",
            USBTC08_ERROR.CONFIG_FAIL: "Missing or corrupted EEPROM.",
            USBTC08_ERROR.NOT_FOUND: "Cannot find enumerated device.",
            USBTC08_ERROR.THREAD_FAIL: "A threading function failed.",
            USBTC08_ERROR.PIPE_INFO_FAIL: "Can not get USB pipe information.",
            USBTC08_ERROR.NOT_CALIBRATED: "No calibration date was found.",
            USBTC08_ERROR.PICOPP_TOO_OLD: "An old picopp.sys driver was found on the system.",
            USBTC08_ERROR.COMMUNICATION: "The PC has lost communication with the device."
        }
        return(d[error])

class USBTC08_UNITS(Enum):
    CENTIGRADE = 0
    FAHRENHEIT = 1
    KELVIN = 2
    RANKINE = 3

class USBTC08_TC_TYPE(Enum):
    DISABLED = " "
    B = "B"
    E = "E"
    J = "J"
    K = "K"
    N = "N"
    R = "R"
    S = "S"
    T = "T"

    @classmethod
    def ordinal(cls, tc_type):
        if isinstance(tc_type, Enum):
            tc_type = tc_type.value
        elif isinstance(tc_type, six.string_types):
            tc_type = tc_type.upper()
        return(ord(tc_type))


class TC08USB(object):
    def __init__(self, dll_path=""):
        dll_path = os.path.dirname(os.path.abspath(__file__)) + os.path.sep        
        os.chdir(dll_path)
        dll_filename = os.path.join(dll_path, 'usbtc08.dll')
        self._dll = ctypes.windll.LoadLibrary(dll_filename)
        
        self._handle = None # handle for device
        
        self._temp = np.zeros( (9,), dtype=np.float32)
        self._overflow_flags = np.zeros( (1,), dtype=np.int16)
        
        self._units = USBTC08_UNITS.CENTIGRADE # 0:C 1:F 2:K 3:RK
        
    def open_unit(self):
        self._handle = self._dll.usb_tc08_open_unit()
        return(self._handle)
        
    def set_mains(self, value=50):
        return(self._dll.usb_tc08_set_mains(self._handle, value))
        
    def set_channel(self, channel, tc_type):
        tc_type = USBTC08_TC_TYPE.ordinal(tc_type)
        return(self._dll.usb_tc08_set_channel(self._handle, channel, tc_type))

    def get_single(self):
        return(self._dll.usb_tc08_get_single(self._handle, self._temp.ctypes.data_as(ctypes.POINTER(ctypes.c_float)), self._overflow_flags.ctypes.data_as(ctypes.POINTER(ctypes.c_float)), self._units.value))

    def close_unit(self):
        return(self._dll.usb_tc08_close_unit(self._handle))
        
    def __getitem__(self, channel):
        return(self._temp[channel])
